var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_core_1 = require("@speedy/node-core");
const chalk_1 = require("chalk");
const htmlhint_1 = require("htmlhint");
const _ = require("lodash");
const utils_1 = require("../../utils");
const lint_html_args_1 = require("./lint-html.args");
const logger = new node_core_1.Logger("Lint HTML");
function lintHtml(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const timer = new node_core_1.Timer(logger);
        let result;
        const mergedOptions = utils_1.args.mergeWithOptions(lint_html_args_1.ARGS, options);
        try {
            timer.start();
            result = yield utils_1.Worker.run(__filename, handleLintHtml.name, mergedOptions);
            return result;
        }
        catch (error) {
            logger.error("", error);
            throw error;
        }
        finally {
            timer.finish();
            if (result && result.length && !mergedOptions.continueOnError) {
                process.exit(1);
            }
        }
    });
}
exports.lintHtml = lintHtml;
/** @internal */
function handleLintHtml(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const configFilePath = utils_1.getConfigFilePath(options.config);
        logger.debug(handleLintHtml.name, `Config file path: ${configFilePath}`);
        const rules = yield node_core_1.fileSystem.readJsonFileAsync(configFilePath);
        const failures = _.filter(yield Promise.all(node_core_1.fileSystem.glob(options.files).map(x => lintFile(x, rules))), x => !_.isEmpty(x.result));
        failures.forEach(x => logger.info(formatFailuresForFile(x)));
        return failures;
    });
}
exports.handleLintHtml = handleLintHtml;
function lintFile(filePath, configData) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.debug(lintFile.name, `filePath: ${filePath}`);
        return {
            result: htmlhint_1.HTMLHint.verify(yield node_core_1.fileSystem.readFileAsync(filePath), configData),
            filePath
        };
    });
}
function formatFailuresForFile(failure) {
    let message = `\n${failure.filePath.replace(node_core_1.fileSystem.getRootPath(), "")}`;
    for (const error of failure.result) {
        message += `\n${chalk_1.default.red(`ERROR: ${error.line}:${error.col}`)}  ${chalk_1.default.gray(`${error.rule.id}`)}  ${chalk_1.default.yellow(error.message)}`;
    }
    return `\n ${message} \n`;
}
