var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_core_1 = require("@speedy/node-core");
const stylelint_1 = require("stylelint");
const utils_1 = require("../../utils");
const lint_sass_args_1 = require("./lint-sass.args");
const logger = new node_core_1.Logger("Lint SASS");
function lintSass(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const timer = new node_core_1.Timer(logger);
        let result;
        const mergedOptions = utils_1.args.mergeWithOptions(lint_sass_args_1.ARGS, options);
        try {
            timer.start();
            result = yield utils_1.Worker.run(__filename, handleLintSass.name, mergedOptions);
            return result;
        }
        catch (error) {
            logger.error("", error);
            throw error;
        }
        finally {
            timer.finish();
            if (result && result.errored && !mergedOptions.continueOnError) {
                process.exit(1);
            }
        }
    });
}
exports.lintSass = lintSass;
/** @internal */
function handleLintSass(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const configFilePath = utils_1.getConfigFilePath(options.config);
        logger.debug(handleLintSass.name, `Config file path: ${configFilePath}`);
        const [configData, files] = yield Promise.all([
            node_core_1.fileSystem.readJsonFileAsync(configFilePath),
            node_core_1.fileSystem.glob(options.files)
        ]);
        const result = yield stylelint_1.lint({
            config: configData,
            formatter: options.formatter,
            files,
            fix: options.fix
        });
        if (result.errored) {
            result.results
                .filter(x => x.errored)
                .forEach(x => logger.info(stylelint_1.formatters.string([x])));
        }
        return result;
    });
}
exports.handleLintSass = handleLintSass;
