var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_core_1 = require("@speedy/node-core");
const _ = require("lodash");
const tslint_1 = require("tslint");
const utils_1 = require("../../utils");
const lint_ts_args_1 = require("./lint-ts.args");
const logger = new node_core_1.Logger("Lint TS");
function lintTs(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const timer = new node_core_1.Timer(logger);
        let result;
        const mergedOptions = utils_1.args.mergeWithOptions(lint_ts_args_1.ARGS, options);
        try {
            timer.start();
            result = yield utils_1.Worker.run(__filename, handleLintTs.name, mergedOptions);
            return result;
        }
        catch (error) {
            logger.error("", error);
            throw error;
        }
        finally {
            timer.finish();
            if (result && result.errorCount && !mergedOptions.continueOnError) {
                process.exit(1);
            }
        }
    });
}
exports.lintTs = lintTs;
/** @internal */
function handleLintTs(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const configFilePath = utils_1.getConfigFilePath(options.config);
        logger.debug(handleLintTs.name, `Config file path: ${configFilePath}`);
        const configData = tslint_1.Configuration.loadConfigurationFromPath(configFilePath);
        if (!configData) {
            throw new Error(`Cannot retrieve 'config' data, path: ${options.config}`);
        }
        const linter = new tslint_1.Linter({
            fix: options.fix,
            formatter: options.formatter
        }, tslint_1.Linter.createProgram(options.program));
        yield Promise.all(node_core_1.fileSystem.glob(options.files).map(x => lintFile(x, configData, linter)));
        const result = linter.getResult();
        if (!_.isEmpty(result.failures)) {
            logger.info(result.output);
        }
        return {
            failureCount: result.failures.length,
            errorCount: result.errorCount,
            fixesCount: result.fixes ? result.fixes.length : 0
        };
    });
}
exports.handleLintTs = handleLintTs;
function lintFile(filePath, configData, linter) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.debug(lintFile.name, `filePath: ${filePath}`);
        linter.lint(filePath, yield node_core_1.fileSystem.readFileAsync(filePath), configData);
    });
}
