Object.defineProperty(exports, "__esModule", { value: true });
const node_core_1 = require("@speedy/node-core");
const _ = require("lodash");
const yargs = require("yargs");
var args;
(function (args) {
    function init() {
        yargs.parse(node_core_1.args.mergedConfigArgsAndProcessArgv());
        set([{
                key: "debug",
                description: "Show debug information",
                boolean: true
            }]);
    }
    args.init = init;
    /**
     * Register command arguments. When `default` value is specified the argument `type` will be inferred.
     */
    function set(argOptions) {
        for (const x of argOptions) {
            yargs.option(x.key, x);
            if (_.isNil(x.default) || x.boolean || x.type || x.number || x.array || x.string) {
                continue;
            }
            if (_.isNumber(x.default)) {
                yargs.number(x.key);
            }
            if (_.isBoolean(x.default)) {
                yargs.boolean(x.key);
            }
            if (_.isString(x.default)) {
                yargs.string(x.key);
            }
            if (_.isArray(x.default)) {
                yargs.array(x.key);
            }
        }
        return yargs.argv;
    }
    args.set = set;
    /**
     * Merges `Arguments` default values with `Options`
     */
    function mergeWithOptions(defaultArgs, options) {
        const defaultOptions = {};
        for (const arg of defaultArgs) {
            if (_.isNil(arg.default)) {
                continue;
            }
            _.set(defaultOptions, arg.key, arg.default);
        }
        // todo: add generic type when issue is solved
        // https://github.com/Microsoft/TypeScript/issues/10727
        return Object.assign({}, defaultOptions, options);
    }
    args.mergeWithOptions = mergeWithOptions;
    args.getAll = () => yargs.argv;
    args.env = args.getAll();
})(args = exports.args || (exports.args = {}));
