Object.defineProperty(exports, "__esModule", { value: true });
const node_core_1 = require("@speedy/node-core");
const child_process_1 = require("child_process");
const lodash_1 = require("lodash");
const path_1 = require("path");
require("./worker.process");
const logger = new node_core_1.Logger("Worker Client");
var Worker;
(function (Worker) {
    const workers = [];
    function run(modulePath, task, parameters) {
        return new Promise((resolve, reject) => {
            const worker = create(task);
            if (!worker) {
                return;
            }
            worker
                .on("message", (message) => {
                if (message.error) {
                    reject(message.error);
                }
                else {
                    resolve(message.resolved);
                }
                kill(worker.pid);
            })
                .on("error", error => logger.error(`task: ${task}, pid: ${worker.pid}`, error))
                .on("exit", code => {
                logger.debug(run.name, `Exit task: ${task}, pid: ${worker.pid}, exitCode: ${code}`);
                if (code > 0) {
                    process.exit(1);
                }
            })
                .send({
                task,
                modulePath,
                parameters
            });
        });
    }
    Worker.run = run;
    function create(task) {
        try {
            const childProcess = child_process_1.fork(path_1.join(__dirname, "worker.process.js"), process.argv);
            workers.push({
                task,
                process: childProcess
            });
            logger.debug(create.name, `task: ${task}, pid: ${childProcess.pid}`);
            return childProcess;
        }
        catch (error) {
            logger.error(`Unable to 'create' worker task: ${task}`, error);
            throw error;
        }
    }
    function kill(pid) {
        const workerIndex = lodash_1.findIndex(workers, x => pid === (x && x.process.pid));
        if (workerIndex < 0) {
            return;
        }
        const worker = workers[workerIndex];
        try {
            worker.process.kill("SIGTERM");
            logger.debug(kill.name, `task: ${worker.task}, pid: ${pid}`);
        }
        catch (error) {
            logger.error(`Unable to 'kill' worker task: ${worker.task}, pid: ${pid}`, error);
        }
        finally {
            delete workers[workerIndex];
        }
    }
})(Worker = exports.Worker || (exports.Worker = {}));
